/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//cardwords_machinestring.hh

/* The class Machine_String is a simple vector of Machine_Chars.
   It provides simple input and output facilities (input delimiter
   white space) and conversion to (const Machine_Char*) and
   (const char *).
*/

#ifndef CARDWORDS_MACHINESTRING_HH
#define CARDWORDS_MACHINESTRING_HH

#include "cardwords_machinechar.hh"
#include <vector>
#include "cardwords_iostream.hh"

class Machine_String : public vector<Machine_Char> {
public:
  Machine_String() {push_back('0');};
  operator const char *() const {return (const char *)begin();};
  operator const Machine_Char *() const {return begin();};

  // convert a number to a string:
  Machine_String &
  operator = (unsigned);

  Machine_String &
  operator = (int);
};

istream & operator >> (istream & istream, Machine_String & str);
ostream & operator << (ostream & ostream, const Machine_String & str);
#endif
