/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_gtk_pileinfo.hh

// CardWords_Gtk_PileInfo is a simple class displaying how many cards are left
// in the pile.

#ifndef CARDWORDS_GTK_PILEINFO_HH
#define CARDWORDS_GTK_PILEINFO_HH

#include <gtk--.h>

class CardWords_Gtk_PileInfo : public Gtk_HBox
{
public:
  CardWords_Gtk_PileInfo(size_t initial_no = 0);
  ~CardWords_Gtk_PileInfo();

  void
  update_no_of_cards(size_t no);
private:
  Gtk_Label * title_label;
  Gtk_Label * count_label;

  size_t no_of_cards;
};

#endif
